import click
from fancykimai.functions.kimai import kimai_request
from fancykimai.classes.click_groups import AliasedGroup

@click.group(name='teams', cls=AliasedGroup)
def teams_group():
    pass

@teams_group.command(name='list')
@click.option('-o', '--output', type=click.Choice(['table', 'json']), default='table', help='Output format')
def list_teams(output: str):
    r = kimai_request('api/teams')
    if output == 'table':
        click.echo('ID\tName')
        for team in r:
            click.echo(f'{team["id"]}\t{team["name"]}')
    else:
        click.echo(r)


import click

class AliasedGroup(click.Group):
    def get_command(self, ctx, cmd_name):
        rv = click.Group.get_command(self, ctx, cmd_name)
        if rv is not None:
            return rv
        matches = [x for x in self.list_commands(ctx)
                   if x.startswith(cmd_name)]
        if not matches:
            return None
        elif len(matches) == 1:
            return click.Group.get_command(self, ctx, matches[0])
        ctx.fail(f"Too many matches: {', '.join(sorted(matches))}")

    def resolve_command(self, ctx, args):
        # always return the full command name
        _, cmd, args = super().resolve_command(ctx, args)
        return cmd.name, cmd, args


from pydantic import BaseModel
from typing import List, Optional
from pydantic import Field

class TimesheetMeta(BaseModel):
    name: str
    value: str

class TimesheetCollection(BaseModel):
    activity: Optional[int]
    project: Optional[int]
    user: Optional[int]
    tags: Optional[List[str]]
    timesheet_id: int = Field(alias="id")
    begin: Optional[str]
    end: Optional[str]
    duration: Optional[int]
    description: Optional[str]
    rate: Optional[float]
    internalRate: Optional[float]
    exported: Optional[bool]
    billable: Optional[bool]
    metaFields: Optional[List[TimesheetMeta]]


from fancykimai.functions.kimai import kimai_request
from fancykimai.functions.config import get_config
from iterfzf import iterfzf

def select_project(ctx, param, value: str) -> str:
    '''
    Select a project from the list of projects in Kimai and return the id
    '''
    default = get_config('project')
    projects = kimai_request('api/projects')
    if value:
        # Check if value is in the id or name of the projects
        for project in projects:
            if value == project['id']:
                return project['id']
            if value == project['name']:
                return project['id']
        # If not found, return an error
        raise ValueError('Project not found')
    else:
        # If no value is given, but there's a default project, return the id
        if default:
            return default
        # If no value is given and there's no default project, prompt the user
        project_names = [f"{project['id']} - {project['name']}" for project in projects]
        selected_project = iterfzf(project_names)
        if selected_project:
            return selected_project.split(' - ')[0]
        else:
            raise ValueError('Project not found')


import click
from fancykimai.functions.kimai import kimai_request
from fancykimai.functions.config import get_config
from fancykimai.functions.timesheets import get_timesheets
from fancykimai.functions.activity import select_activity
from fancykimai.functions.project import select_project
import rich
from rich import table, console
import datetime
import json
from iterfzf import iterfzf
from fancykimai.classes.click_groups import AliasedGroup


@click.group(name="timesheets", cls=AliasedGroup)
def timesheets_group():
    """
    Timesheet commands
    """

    pass


@timesheets_group.command(name="list")
@click.option(
    "-o",
    "--output",
    type=click.Choice(["table", "json"]),
    default="table",
    help="Output format",
)
@click.option("-a", "--active", help="Only show active timesheets", is_flag=True)
@click.option("-p", "--project", type=int, help="Filter by project ID")
@click.option("-u", "--user", type=int, help="Filter by user ID")
@click.option("-b", "--begin", type=str, help="Filter by begin date")
@click.option("-e", "--end", type=str, help="Filter by end date")
@click.option("--show-costs", help="Show costs", is_flag=True)
@click.option("--show-total", help="Show total", is_flag=True)
def list_timesheets(
    output: str,
    active: bool,
    project: int,
    user: int,
    begin: str,
    end: str,
    show_costs: bool,
    show_total: bool,
):
    """
    List timesheets
    """

    # format the begin and end dates
    if begin:
        begin = datetime.datetime.strptime(begin, "%Y-%m-%d").strftime(
            "%Y-%m-%dT00:00:00"
        )
    if end:
        end = datetime.datetime.strptime(end, "%Y-%m-%d").strftime("%Y-%m-%dT23:59:59")
    if not begin and not end:
        # begin = sunday this week
        begin = datetime.datetime.now().replace(
            hour=0, minute=0, second=0, microsecond=0
        )
        begin -= datetime.timedelta(days=begin.weekday())
        # end = saturday this week
        end = begin + datetime.timedelta(days=6)
        begin = begin.strftime("%Y-%m-%dT00:00:00")
        end = end.strftime("%Y-%m-%dT23:59:59")

    r = kimai_request(
        "api/timesheets",
        data={
            "project": project,
            "user": user,
            "begin": begin,
            "end": end,
            "active": (1 if active else None),
        },
    )
    if output == "table":
        columns = [
            {"column": "ID", "response_key": "id", "function": str, "style": "cyan"},
            {
                "column": "Activity",
                "response_key": "activity",
                "function": str,
                "style": "magenta",
            },
            {
                "column": "Project",
                "response_key": "project",
                "function": str,
                "style": "green",
            },
            {
                "column": "Start",
                "response_key": "begin",
                "function": lambda date: (
                    datetime.datetime.strptime(date, "%Y-%m-%dT%H:%M:%S%z").strftime(
                        "%Y-%m-%d %H:%M:%S"
                    )
                    if date
                    else "Active"
                ),
                "style": "yellow",
            },
            {
                "column": "End",
                "response_key": "end",
                "function": lambda date: date if date else "Active",
                "style": "blue",
            },
            {
                "column": "Duration (Hours)",
                "response_key": "duration",
                "function": lambda s: str(s / 3600),
                "style": "red",
            },
            {
                "column": "User",
                "response_key": "user",
                "function": str,
                "style": "cyan",
            },
            {
                "column": "Description",
                "response_key": "description",
                "function": str,
                "style": "magenta",
            },
        ]
        if show_costs or show_total:
            columns.append(
                {
                    "column": "Cost",
                    "response_key": "rate",
                    "function": lambda s: str(s),
                    "style": "green",
                }
            )
        # define a table using rich to print out the values
        rich_table = table.Table(title="Timesheets")
        totals = {"duration": 0, "rate": 0}
        for column in columns:
            rich_table.add_column(column["column"], style=column["style"])
        for timesheet in r:
            rich_table.add_row(
                *[
                    column["function"](timesheet[column["response_key"]])
                    for column in columns
                ]
            )
            totals["duration"] += float(timesheet["duration"])
            totals["rate"] += float(timesheet["rate"])
        if show_total:
            # add divider for the totals
            rich_table.add_row("", "", "", "", "", "", "", "")
            # add a row for the totals
            rich_table.add_row(
                "Total",
                "",
                "",
                "",
                "",
                str(totals["duration"] / 3600),
                "",
                "",
                str(totals["rate"]),
            )
        rich_console = console.Console()
        rich_console.print(rich_table)
    else:
        click.echo(rich.print(json.dumps(r)))


@timesheets_group.command(name="start")
# get the project option from the configuration file if there's any. Otherwise it's required
@click.option(
    "-p",
    "--project",
    type=int,
    required=True,
    help="Project ID",
    default=get_config("project"),
)
@click.option(
    "-a",
    "--activity",
    type=int,
    required=True,
    help="Activity ID",
    default=get_config("activity"),
)
@click.option("-d", "--description", type=str, required=True, help="Description")
def start_timesheet(project: int, activity: int, description: str):
    """
    Start a new timesheet entry
    """
    begin = datetime.datetime.now().strftime("%Y-%m-%dT%H:%M:%S")
    try:
        r = kimai_request(
            "api/timesheets",
            method="post",
            data={
                "project": project,
                "activity": activity,
                "description": description,
                "begin": begin,
            },
        )
        click.echo(f"Started timesheet {r['id']} for project {r['project']} successfully.")
    except Exception as e:
        click.echo(f"Error: {e}")


@timesheets_group.command(name="stop")
@click.option("-i", "--id", type=int, required=False, help="Timesheet ID")
def stop_timesheet(id: int):
    """
    Stop a timesheet entry
    """

    if id is None:
        # Get the active timesheets to select
        timesheets = get_timesheets(data={"active": 1})
        if len(timesheets) > 0:
            selected_timesheet = iterfzf(
                [f"{timesheet.timesheet_id} - {timesheet.description}".replace("\r\n", "") for timesheet in timesheets],
                multi=False,
                prompt="Select a timesheet: ",
            )
            if selected_timesheet:
                id = selected_timesheet.split()[0]
            else:
                click.echo("No timesheet selected.")
                raise click.Abort("No timesheet selected.")
    end = datetime.datetime.now().strftime("%Y-%m-%dT%H:%M:%S")
    try:
        r = kimai_request(f"api/timesheets/{id}/stop", method="get", data={"end": end})
        click.echo(f"Timesheet {id} stopped successfully.")
    except Exception as e:
        click.echo(f"Error: {e}")


@timesheets_group.command(name="delete")
@click.argument("timesheet_id", type=int, required=False)
def delete_timesheet(timesheet_id: int):
    """
    Delete a timesheet entry
    """

    if timesheet_id is None:
        # get all timesheets
        timesheets = get_timesheets()
        if len(timesheets) > 0:
            selected_timesheet = iterfzf(
                [f"{timesheet.timesheet_id} - {timesheet.begin} {timesheet.description}".replace("\r\n", "") for timesheet in timesheets],
                multi=False,
                prompt="Select a timesheet: ",
            )
            if selected_timesheet:
                timesheet_id = selected_timesheet.split()[0]
            else:
                click.echo("No timesheet selected.")
                raise click.Abort("No timesheet selected.")
        else:
            click.echo("No timesheets found.")
            raise click.Abort("No timesheets found.")
    try:
        r = kimai_request(f"api/timesheets/{timesheet_id}", method="delete")
        click.echo(f"Timesheet {timesheet_id} deleted successfully.")
    except Exception as e:
        click.echo(f"Error: {e}")



@timesheets_group.command(name="set")
@click.option(
    "-p",
    "--project",
    type=int,
    help="Project ID",
    default=get_config("project"),
    callback=select_project,
)
@click.option(
    "-a",
    "--activity",
    type=int,
    help="Activity ID",
    default=get_config("activity"),
    callback=select_activity,
)
@click.option("-d", "--description", type=str, required=False, help="Description", prompt="Please enter a description")
@click.option("-b", "--begin", type=str, required=True, help="Begin date", default=datetime.datetime.now().strftime("%Y-%m-%d"))
@click.option("-e", "--end", type=str, required=False, help="End date")
@click.option("-h", "--hours", type=float, required=False, help="Hours to be set. When set, ignores the end date and if no begin time is set, sets it to 09:00")
def set_timesheet(project: int, activity: int, description: str, begin: str, end: str, hours: float):
    """
    Set a timesheet entry
    """
    
    # Check if the hours are set on the dates
    begin_datetime = datetime.datetime.strptime(begin, "%Y-%m-%d")
    if end:
        end_datetime = datetime.datetime.strptime(end, "%Y-%m-%d")
    else:
        end_datetime = begin_datetime
        end = begin
    if begin_datetime > end_datetime:
        click.echo("Begin date cannot be after end date.")
        raise click.Abort("Begin date cannot be after end date.")
    if not hours:
        # ask for the time to be set
        begin_time = click.prompt("Begin time", type=str, default="00:00")
        end_time = click.prompt("End time", type=str, default="23:59")
    else:
        begin_time = "09:00"
        # add `hours` to 9 AM to get the end time
        am9 = datetime.datetime.strptime(begin, "%Y-%m-%d").replace(hour=9, minute=0)
        end_time = (am9 + datetime.timedelta(hours=hours)).strftime("%H:%M")
    begin = f"{begin}T{begin_time}:00"
    end = f"{end}T{end_time}:00"
    # Check if the begin and end dates are valid
    begin_datetime = datetime.datetime.strptime(begin, "%Y-%m-%dT%H:%M:%S")
    end_datetime = datetime.datetime.strptime(end, "%Y-%m-%dT%H:%M:%S")
    if begin_datetime > end_datetime:
        click.echo("Begin date cannot be after end date.")
        raise click.Abort("Begin date cannot be after end date.")
    r = kimai_request(
        "api/timesheets",
        method="post",
        data={
            "project": project,
            "activity": activity,
            "description": description,
            "begin": datetime.datetime.strftime(begin_datetime, "%Y-%m-%dT%H:%M:%S"),
            "end": datetime.datetime.strftime(end_datetime, "%Y-%m-%dT%H:%M:%S"),
        },
    )
    click.echo(r)


if __name__ == "__main__":
    delete_timesheet()


import os
import json

def get_config(key) -> str:
    config_file = os.path.expanduser(f"~/.config/fancykimai/config.json")
    if os.path.exists(config_file):
        with open(config_file, "r") as f:
            configuration = json.loads(f.read())
        # Get the selected context
        context = configuration.get("selected_context", "default")
        context_config = next((c for c in configuration["contexts"] if c["name"] == context), None)
        if context_config:
            return context_config.get(key, None)
        else:
            return None
    else:
        return None

def set_config(key: str, value: str, context: str = None):
    config_file = os.path.expanduser(f"~/.config/fancykimai/config.json")
    if os.path.exists(config_file):
        with open(config_file, "r") as f:
            configuration = json.loads(f.read())
        if not context:
            context = configuration.get("selected_context", "default")
        context_config = next((c for c in configuration["contexts"] if c["name"] == context), None)
        if context_config:
            context_config[key] = value
        else:
            configuration["contexts"].append({"name": context, key: value})
        with open(config_file, "w") as f:
            f.write(json.dumps(configuration, indent=4))
    else:
        config_dir = os.path.dirname(config_file)
        if not os.path.exists(config_dir):
            os.makedirs(config_dir)
        configuration = {
            "selected_context": "default",
            "contexts": [{"name": "default", key: value}]
        }
        with open(config_file, "w") as f:
            f.write(json.dumps(configuration, indent=4))

def unset_config(key: str, context: str = None):
    config_file = os.path.expanduser(f"~/.config/fancykimai/config.json")
    if os.path.exists(config_file):
        with open(config_file, "r") as f:
            configuration = json.loads(f.read())
        if not context:
            context = configuration.get("selected_context", "default")
        context_config = next((c for c in configuration["contexts"] if c["name"] == context), None)
        if context_config:
            context_config.pop(key, None)
            with open(config_file, "w") as f:
                f.write(json.dumps(configuration, indent=4))
        else:
            raise ValueError("Context not found")
    else:
        raise ValueError("Configuration file not found")


import click
from fancykimai.commands import commands
from fancykimai.classes.click_groups import AliasedGroup

@click.group(cls=AliasedGroup)
def cli():
    pass

for command in commands:
    cli.add_command(command)

if __name__ == '__main__':  
    cli()


from .login import kimai_login
from .projects import projects_group
from .activities import activities_group
from .customers import customers_group
from .teams import teams_group
from .timesheets import timesheets_group
from .config import config_group

commands = [
    kimai_login,
    projects_group,
    activities_group,
    customers_group,
    teams_group,
    timesheets_group,
    config_group,
]


import click
from fancykimai.functions.kimai import kimai_request
from fancykimai.functions.config import set_config
from rich import table, console
import json
from iterfzf import iterfzf
from fancykimai.classes.click_groups import AliasedGroup

@click.group(name='projects', cls=AliasedGroup)
def projects_group():
    '''
    Project commands
    '''
    pass

@projects_group.command(name='list')
@click.option('-o', '--output', type=click.Choice(['table', 'json']), default='table', help='Output format')
def list_projects(output: str):
    '''
    List projects
    '''
    r = kimai_request('api/projects')
    if output == 'table':
        columns = [
            {'column': "ID", 'response_key': 'id', 'function': str, 'style': 'cyan'},
            {'column': "Name", 'response_key': 'name', 'function': str, 'style': 'magenta'},
        ]
        rich_table = table.Table(title="Projects")
        for column in columns:
            rich_table.add_column(column['column'], style=column['style'])
        for project in r:
            rich_table.add_row(*[column['function'](project[column['response_key']]) for column in columns])
        rich_console = console.Console()
        rich_console.print(rich_table)
    else:
        click.echo(json.dumps(r))

@projects_group.command(name='select')
@click.argument('project_id', type=int, required=False)
def select_project(project_id: int):
    '''
    Select a project
    '''
    if project_id is None:
        # get all projects
        projects = kimai_request('api/projects')
        # if fzf is installed use it to select a project
        if len(projects) > 0:
            selected_project = iterfzf([f"{project['id']} - {project['name']}" for project in projects], multi=False, prompt="Select a project: ")
            if selected_project is not None:
                project_id = selected_project.split()[0]
            set_config('project', project_id)
            click.echo(f"Selected project: {project_id}")
        else:
            click.Abort("No projects found.")
    else: 
        r = kimai_request('api/projects/' + str(project_id))
        set_config('project', project_id)
        click.echo(f"Selected project: {r['name']}")


import click
import os
from fancykimai.functions.config import get_config, set_config, unset_config
from fancykimai.classes.click_groups import AliasedGroup

@click.group(name="config", cls=AliasedGroup)
def config_group():
    """
    Configuration commands
    """
    pass

@config_group.command()
def show():
    """
    Show the configuration file
    """
    config_file = os.path.expanduser("~/.config/fancykimai/config.json")
    if os.path.exists(config_file):
        with open(config_file, "r") as f:
            config_data = f.read()
            click.echo(config_data)
    else:
        click.echo("Configuration file not found.")

@config_group.command()
@click.argument("key")
@click.argument("value")
@click.option("--context", help="Set the configuration value for a specific context", default=None, required=False)
def set(key, value, context):
    """
    Set a configuration value
    """
    set_config(key, value, context=context)
    click.echo(f"Set {key} to {value}")

@config_group.command()
@click.argument("key")
@click.option("--context", help="Unset the configuration value for a specific context", default=None, required=False)
def get(key, context):
    """
    Get a configuration value
    """
    click.echo(get_config(key, context=context))

@config_group.command()
@click.argument("key")
@click.option("--context", help="Unset the configuration value for a specific context", default=None, required=False)
def unset(key,  context):
    """
    Unset a configuration value
    """
    unset_config(key, context=context)
    click.echo(f"Unset {key}")


import click
from fancykimai.functions.kimai import kimai_request
from rich import table, console
import json
from fancykimai.classes.click_groups import AliasedGroup

@click.group(name='customers', cls=AliasedGroup)
def customers_group():
    """
    Customer commands
    """
    pass

@customers_group.command(name='list')
@click.option('-o', '--output', type=click.Choice(['table', 'json']), default='table', help='Output format')
def list_customers(output: str):
    """
    List customers
    """
    r = kimai_request('api/customers')
    if output == 'table':
        columns = [
            {'column': "ID", 'response_key': 'id', 'function': str, 'style': 'cyan'},
            {'column': "Name", 'response_key': 'name', 'function': str, 'style': 'magenta'},
        ]
        rich_table = table.Table(title="Customers")
        for column in columns:
            rich_table.add_column(column['column'], style=column['style'])
        for customer in r:
            rich_table.add_row(*[column['function'](customer[column['response_key']]) for column in columns])
        rich_console = console.Console()
        rich_console.print(rich_table)
    else:
        click.echo(json.dumps(r))


import requests
import keyring
import urllib
from fancykimai.functions.config import get_config

def get_context(context_name: str) -> dict:
    keyring_user = keyring.get_password(f'kimai:{context_name}', 'user')
    keyring_password = keyring.get_password(f'kimai:{context_name}', 'password')
    keyring_url = keyring.get_password(f'kimai:{context_name}', 'url')
    if keyring_user is None or keyring_password is None or keyring_url is None:
        raise ValueError('Context not found')
    return {'user': keyring_user, 'password': keyring_password, 'url': keyring_url}

def kimai_request(path, method='GET', data=None, headers=None, base_url='default', context_name='default') -> dict:
    # check if keyring is set
    try:
        if is_context_there := get_config('context'):
            context_name = is_context_there
        context_values = get_context(context_name)
    except ValueError:
        context_values = {}
    # if keyring is not set and the call doesn't come from kimai_login, return an error
    if context_values.get('user') is None and path != 'api/ping':
        raise ValueError('Authentication not set. Use "kimai login" to set your authentication.')
    if base_url == 'default':
        if context_values.get('url') is None:
            raise ValueError('Kimai URL not set. Use "kimai login" to set your authentication.')
        base_url = context_values.get('url')
    url = urllib.parse.urljoin(base_url, path)
    if headers is None:
        headers = {'Content-Type': 'application/json'}
    if path != 'api/ping':
        headers['X-AUTH-USER'] = context_values.get('user')
        headers['X-AUTH-TOKEN'] = context_values.get('password')
    if method.upper() == 'GET':
        if data is not None:
            r = requests.get(url, headers=headers, params=data)
        else:
            r = requests.get(url, headers=headers)
    elif method.upper() == 'POST':
        r = requests.post(url, headers=headers, json=data)
    elif method.upper() == 'PUT':
        r = requests.put(url, headers=headers, json=data)
    elif method.upper() == 'DELETE':
        r = requests.delete(url, headers=headers)
        if r.status_code == 204:
            return {'status': 'success', 'message': 'Deleted'}
    else:
        raise ValueError('Method not supported')
    r.raise_for_status()

    return r.json()




from fancykimai.functions.kimai import kimai_request
from fancykimai.classes.kimai_types import TimesheetCollection
from pydantic import parse_obj_as
from typing import List

def get_timesheets(data=None) -> List[TimesheetCollection]:
    timesheets = kimai_request("api/timesheets", data=data)
    return [parse_obj_as(TimesheetCollection, timesheet) for timesheet in timesheets]




import click
import keyring
from fancykimai.functions.kimai import kimai_request
from fancykimai.functions.config import set_config

def set_context(context_name: str, user: str, api_key: str, url: str):
    keyring.set_password(f'kimai:{context_name}', 'user', user)
    keyring.set_password(f'kimai:{context_name}', 'password', api_key)
    keyring.set_password(f'kimai:{context_name}', 'url', url)

@click.command(name='login')
@click.option('-u', '--user', prompt='Your Kimai username', help='Your Kimai username')
@click.option('-p', '--api-key', prompt='Your Kimai API secret', help='Your API secret', hide_input=True)
@click.option('-k', '--url', prompt='Your Kimai URL', help='Your Kimai URL')
@click.option('-c', '--context-name', default='default', help='The context name to use for this authentication')
def kimai_login(user, api_key, url, context_name):
    # check if the authentication works
    r = kimai_request('api/ping', base_url=url, headers={'X-AUTH-USER': user, 'X-AUTH-TOKEN': api_key})
    if r['message'] != 'pong':
        raise ValueError('Authentication failed')
    # set the user and password in the keyring
    set_context(context_name, user, api_key, url)
    set_config('context', context_name)
    click.echo('Authentication successful')



from fancykimai.functions.kimai import kimai_request
from fancykimai.functions.config import get_config
from iterfzf import iterfzf

def select_activity(ctx, param, value: str) -> str:
    '''
    Select an activity from the list of activities in Kimai and return the id
    '''
    default = get_config('activity')
    activities = kimai_request('api/activities')
    if value:
        # Check if value is in the id or name of the activities
        for activity in activities:
            if value == activity['id']:
                return activity['id']
            if value == activity['name']:
                return activity['id']
        # If not found, return an error
        raise ValueError('Activity not found')
    else:
        # If no value is given, but there's a default activity, return the id
        if default:
            return default
        # If no value is given and there's no default activity, prompt the user
        activity_names = [f"{activity['id']} - {activity['name']}" for activity in activities]
        selected_activity = iterfzf(activity_names)
        if selected_activity:
            return selected_activity.split(' - ')[0]
        else:
            raise ValueError('Activity not found')

